unit test_main;

{$mode objfpc}{$H+}

interface

uses
  Forms, Classes, Dialogs, Controls, Graphics, SysUtils, JSONPropStorage,
  ExtCtrls, IdGlobal, StdCtrls, IdUDPServer, IdSocketHandle;

{$i 'defaults.inc'}

type

  { TTestForm }

  TTestForm = class(TForm)
    AlwaysTop : TCheckBox;
    CloseAll : TCheckBox;
    TestRecieved : TLabel;
    Send :   TButton;
    ModuleName : TComboBox;
    UDP :    TIdUDPServer;
    Msg :    TComboBox;
    Label2 : TLabel;
    Label3 : TLabel;
    FlowPanel1 : TFlowPanel;
    FP :     TFlowPanel;
    CtrlGroup : TGroupBox;
    JS :     TJSONPropStorage;
    Label1 : TLabel;
    Param :  TEdit;
    procedure AlwaysTopChange(Sender : TObject);
    procedure FormCreate(Sender : TObject);
    procedure FormDestroy(Sender : TObject);
    procedure SendClick(Sender : TObject);
    procedure MsgChange(Sender : TObject);
    procedure UDPUDPRead(AThread : TIdUDPListenerThread; const AData : TIdBytes; ABinding : TIdSocketHandle);
  private
    procedure UpdateModuleNames(N : string);
    procedure Broadcast(S : string);
  public

  end;

var
  TestForm : TTestForm;

implementation

uses
  StrUtils;

{$R *.frm}

{ TTestForm }

procedure TTestForm.AlwaysTopChange(Sender : TObject);
begin
  if AlwaysTop.Checked then
    FormStyle := fsSystemStayOnTop
  else
    FormStyle := fsNormal;
end;

procedure TTestForm.Broadcast(S : string);
begin
  UDP.Broadcast(AnsiToUtf8(S), cPortCtrl);
end;

procedure TTestForm.FormCreate(Sender : TObject);
begin
  Send.Enabled := Msg.Text <> '';
  Broadcast(cMsgRollCall);
end;

procedure TTestForm.FormDestroy(Sender : TObject);
begin
  if CloseAll.Checked then
    Broadcast(cMsgQuit);
end;

procedure TTestForm.SendClick(Sender : TObject);
var
  S : string;
  R : string;
begin
  S := Msg.Text;
  if ModuleName.Text <> 'BROADCAST' then begin
    S += ':' + ModuleName.Text;
  end else begin
    if Param.Text <> '' then
      S += ':';
  end;
  if Param.Text <> '' then
    S += ',' + Param.Text;
  Broadcast(S);
end;

procedure TTestForm.MsgChange(Sender : TObject);
begin
  Send.Enabled := Msg.Text <> '';
end;

procedure TTestForm.UDPUDPRead(AThread : TIdUDPListenerThread; const AData : TIdBytes; ABinding : TIdSocketHandle);
var
  S, Cmd, Adr, Par : String;
  i : integer;
begin
  if ABinding.Port = cPortCtrl then begin
    S := BytesToString(aData);
    // Test view readed data
    TestRecieved.Caption := S;
    Cmd := ExtractWord(1, S, [':']);
    Adr := ExtractWord(2, S, [':']);
    i := Length(Cmd);
    if Adr <> '' then
      i += length(Adr) + 2;
    Par := copy(S, i, MaxInt);
    Delete(Par, 1, 1);
    case Cmd of
      cMsgLoaded : UpdateModuleNames(Adr);
      cMsgUnloaded : with ModuleName.Items do
          Delete(IndexOf(Adr));
    end;
  end;
end;

procedure TTestForm.UpdateModuleNames(N : string);
begin
  with ModuleName.Items do
    if IndexOf(N) = -1 then
      Add(N);
end;

end.
